import React, { useCallback, useEffect, useMemo, useState } from 'react';
import { Button, Col, Form, Row } from 'react-bootstrap';
import { ChevronBarLeft, ChevronBarRight, ChevronLeft, ChevronRight } from 'react-bootstrap-icons';
import { SetPaginationFilter, getRowOptions, getRowRangeText } from './rowcontrols.helper';
import { RemoveFilterFromArray } from '../common';
/** Unique value for filters applied by this component. */
var FILTER_VALUE = 'rowcontrols';
/**
 * This is used to create table paigniation controls at the bottom of the table
 * @param {RBTRowControlOptions} param0
 * @returns
 */
export var RBTRowControls = function (_a) {
    var rowsPerPage = _a.rowsPerPage, data = _a.data, name = _a.name, varient = _a.varient, handleDisplayedRows = _a.handleDisplayedRows;
    var styleVarient = useMemo(function () { return varient !== null && varient !== void 0 ? varient : 'dark'; }, [varient]);
    var tableName = useMemo(function () {
        var result = 'Row Controls';
        if (name) {
            if (name.length > 0) {
                result = name + ' ' + result;
            }
        }
        return result;
    }, [name]);
    /** How Many Table rows should be displayed? The default is 5 */
    var _b = useState(rowsPerPage !== null && rowsPerPage !== void 0 ? rowsPerPage : 5), paginationRows = _b[0], setPaginationRows = _b[1];
    /** What is the current position of displayed rows within the table (e.g. if there are 5 rows per page and 10 records, position could be 0, 5)*/
    var _c = useState(0), tablePosition = _c[0], setTablePosition = _c[1];
    /**
     * If a callback function has been supplied his will update the data to indicate if row controls should be applied to the data.
     *
     * @param position the position we should be centered on.
     * @param rowsPerPage  the number of rows we want to see on a 'page'
     * @param rows  the rows to be processed
     * @param rowsToMark allows us to control how much is displayed
     * @returns processed RBTRows
     */
    var updateDisplayedRows = useCallback(function (position, rowsPerPage, rows, rowsToMark) {
        if (rowsToMark === void 0) { rowsToMark = 0; }
        if (typeof handleDisplayedRows === 'function') {
            var original = JSON.stringify(data);
            var updatedRows = SetPaginationFilter(position, rowsPerPage, rows, rowsToMark);
            var updated = JSON.stringify(updatedRows);
            // Use strings to check if we have added/removed rowcontrol filters to anytning
            if (original !== updated) {
                handleDisplayedRows(updatedRows);
            }
        }
    }, []);
    /**
     * This is supplied a new table position, it will iterate only data from that postion (to the range endpoint)
     * @param tablePos the new table position
     */
    var handleTablePosition = useCallback(function (tablePos) {
        if (tablePosition !== tablePos) {
            setTablePosition(tablePos);
        }
        updateDisplayedRows(tablePos, paginationRows, data);
    }, [data]);
    /** Works out how many rows are visible to the user, to create a row total. */
    var numRows = useMemo(function () {
        var result = 0;
        if (data) {
            data.forEach(function (row) {
                if (row) {
                    if (row.filters.length === 0) {
                        result++;
                    }
                    else {
                        // Check it isn't this component filtering the value, if it is we should include it
                        // within the count
                        var filters = RemoveFilterFromArray(FILTER_VALUE, row.filters);
                        if (filters.length === 0) {
                            result++;
                        }
                    }
                }
            });
        }
        return result;
    }, [data]);
    /** Gets the table sizes based on the number of rows of data.*/
    var rowOptions = useMemo(function () { return getRowOptions(numRows); }, [data]);
    /** Generates the text which sits next to the drop down to indicate to the user, */
    var rangeText = useMemo(function () { return getRowRangeText(numRows, paginationRows, tablePosition); }, [numRows, paginationRows, tablePosition]);
    /**
     * Called as the drop down can contain numbers and string values, so this correctly converts
     * them as appropriate.
     *
     * @param {string} value the Rows Per Page value we want to update the table to
     */
    var handleTablePagination = function (value) {
        if (value) {
            if (value === 'All') {
                setPaginationRows(numRows);
                updateDisplayedRows(0, numRows, data);
            }
            else {
                var convertedValue = Number(value);
                if (isNaN(convertedValue)) {
                    console.log('handleTablePagination: Invalid Value has been supplied ' + value);
                }
                else {
                    setPaginationRows(convertedValue);
                    updateDisplayedRows(tablePosition, convertedValue, data, convertedValue);
                }
            }
        }
    };
    useEffect(function () {
        handleTablePosition(tablePosition);
    }, [data, tablePosition]);
    return (React.createElement(Row, { className: 'justify-content-end mx-1 my-1' },
        React.createElement(Col, { xs: 'auto' },
            React.createElement("p", { className: 'pt-2' }, "Rows per page")),
        React.createElement(Col, { xs: 'auto' },
            React.createElement(Form.Select, { "aria-expanded": false, "aria-haspopup": 'listbox', "aria-label": tableName + ' Set Rows Per Table Page', className: 'mx-1 my-1', onChange: function (e) { return handleTablePagination(e.currentTarget.value); } }, rowOptions.map(function (option) {
                return (React.createElement("option", { role: 'menuitem', key: 'table.dropdown.' + option }, option));
            }))),
        React.createElement(Col, { xs: 'auto' },
            React.createElement("p", { className: 'pt-2' }, rangeText)),
        numRows > paginationRows ? (React.createElement(Col, { xs: 'auto' },
            React.createElement(Button, { "aria-label": tableName + ' Move to first page of table results', className: 'mx-1 my-1 px-2', onClick: function () { return handleTablePosition(0); }, disabled: tablePosition === 0, variant: styleVarient },
                React.createElement(ChevronBarLeft, null)),
            React.createElement(Button, { "aria-label": tableName + ' Move to previous page of table results', className: 'mx-1 my-1 px-2', onClick: function () { return handleTablePosition(tablePosition - paginationRows); }, disabled: tablePosition - paginationRows < 0, variant: styleVarient },
                React.createElement(ChevronLeft, null)),
            React.createElement(Button, { "aria-label": tableName + ' Move to next page of table results', className: 'mx-1 my-1 px-2', onClick: function () { return handleTablePosition(tablePosition + paginationRows); }, disabled: tablePosition + paginationRows >= numRows, variant: styleVarient },
                React.createElement(ChevronRight, null)),
            React.createElement(Button, { "aria-label": tableName + ' Move to last page of table results', className: 'mx-1 my-1 px-2', onClick: function () { return handleTablePosition(numRows - paginationRows); }, disabled: numRows - paginationRows <= tablePosition, variant: styleVarient },
                React.createElement(ChevronBarRight, null)))) : null));
};
//# sourceMappingURL=rowcontrols.components.js.map